//
// ChessGame.cpp
//

#include "ChessGame.h"
#include "chessdef.h"
#include <cstdlib>
#include <cstring>
#include "ChessPlayer.h"
#include "ScriptingEngine.h"
#include "ChessMove.h"
#include "ChessBoard.h"

#include "debug.h"


///////////////////////////////////////////////////////////////////////////////

ChessGame::ChessGame() 
  : baseGamePath("players/"),
    commonGamePath("common/"),
    whiteClassName("WhitePlayerAI"), 
    blackClassName("BlackPlayerAI")    	  
{
  scriptingEngine = NULL;		
  whitePlayer = NULL;
  blackPlayer = NULL;
  reset();
}

///////////////////////////////////////////////////////////////////////////////

ChessGame::~ChessGame()
{
  dprintf(("entered ~ChessGame()\n"));
  reset();
  dprintf(("leaving ~ChessGame()\n"));
}

///////////////////////////////////////////////////////////////////////////////

void ChessGame::reset()
{
  SAFE_DELETE(whitePlayer);
  SAFE_DELETE(blackPlayer);
  SAFE_DELETE(scriptingEngine);
  OK = false;
}

///////////////////////////////////////////////////////////////////////////////

bool ChessGame::init(int argc, char **argv)
{
  parseCommandLineArgs(argc, argv);
  
  chessBoard = new ChessBoard();
  chessBoard->initialize();
  
  scriptingEngine = new ScriptingEngine();
  if(!scriptingEngine->init(whiteGamePath, blackGamePath))
    return false;
  
  whitePlayer = new ChessPlayer("whitey");	
  if(!whitePlayer->initialize(scriptingEngine->
			      newScriptController(whitePlayerClass)))
    return false;
  
  blackPlayer = new ChessPlayer("black");	
  if(!blackPlayer->initialize(scriptingEngine->
			      newScriptController(blackPlayerClass)))
    return false;

  return OK = true;
}

///////////////////////////////////////////////////////////////////////////////

void ChessGame::parseCommandLineArgs(int argc, char **argv)
{

  // instead of -game parse for -black and -white, can 
  // accept freechess.org(netchess), gnuchess, human, etc.
  // ...what to do if the custom game does not exist?
  int i = 1;
  while(i < argc){
    char *arg = argv[i];
    arg++; // skip '-'
    switch(arg[0]){
    case 'w':  // white custom game path
      arg = argv[++i];
      dprintf(("white custom game path: %s\n", arg));
      whiteGamePath = baseGamePath + string(arg);			
      break;
    case 'b': // black custom game path
      arg = argv[++i];
      dprintf(("black custom game path: %s\n", arg));
      blackGamePath = baseGamePath + string(arg);			
      break;
    default: 
      break;
    }
    arg = argv[++i];
  }	
  
  // default args 
  //if(blackGamePath[0] == NULL){
  if(blackGamePath.length() == 0){
    blackGamePath = baseGamePath + "human";		
  }
  
  //if(whiteGamePath[0] == NULL){
  if(whiteGamePath.length() == 0){
    whiteGamePath = baseGamePath + "human";		
  }
  
  whitePlayerClass = whiteGamePath + "/" + whiteClassName;
  blackPlayerClass = blackGamePath + "/" + blackClassName;  
}

///////////////////////////////////////////////////////////////////////////////

void ChessGame::start()
{		
  if(!isOK()) return;
  gameLoop();	
  printf("quitting game...\n");
}

///////////////////////////////////////////////////////////////////////////////
/*
** TODO: 1. make this more general for any two game types
**	 2. use '.' at beginning human input string to indicate that the 
**          string is a message to be sent directly to whatever chess AI
**	    is being used, should ignore command or repory if it doesn't 
**          understand
*/
void ChessGame::gameLoop()
{
  if(!isOK()) return;
  dprintf(("entered ChessGame::gameLoop()\n"));
  ChessMove *blackMove, *whiteMove;
  blackMove = whiteMove = NULL;
  while(true){
    printf("\n");
    chessBoard->print();
    
    printf("\n");
    whiteMove = whitePlayer->getNextMove(blackMove);
    if(whiteMove == NULL) return;
    
    dprintf(("got white move\n"));
    dprintf(("\nwhite move is %s\n\n", whiteMove->toAlgeabraicMove()));
    printf("\nwhite move is %s\n\n", whiteMove->toAlgeabraicMove());
    
    if(whiteMove->getIsCheckMate()){
      printf("checkmate ........ white wins\n");
      return;
    }
    
    chessBoard->movePiece(whiteMove, false);
    chessBoard->print();
    
    printf("\nthinking....\n");
    blackMove = blackPlayer->getNextMove(whiteMove);
    if(blackMove == NULL) return;
    
    printf("\nblack move is %s\n\n", blackMove->toAlgeabraicMove());
    
    if(whiteMove->getIsCheckMate()){
      printf("checkmate ........ white wins\n");
      return;
    }
    
    
    chessBoard->movePiece(blackMove, true);
  }
}
