//
// ScriptingEngine.cpp
//

#include "chessdef.h"
#include "ScriptingEngine.h"
#include "JavaCPP.h"
#include "debug.h"
#include "ScriptController.h"
#include "crossOS.h"

#ifdef OS_UNIX
#include <unistd.h>
#endif

///////////////////////////////////////////////////////////////////////////////

ScriptingEngine::ScriptingEngine()
{
  dprintf(("entered ScriptingEngine()\n"));
  jcpp = NULL;
  reset();
  dprintf(("entered ScriptingEngine()\n"));
}

///////////////////////////////////////////////////////////////////////////////


ScriptingEngine::~ScriptingEngine()
{
  dprintf(("entered ~ScriptEngine()\n"));
  reset();
  dprintf(("leaving ~ScriptEngine()\n"));
}

///////////////////////////////////////////////////////////////////////////////

// reset class and all members
// NOTE: pointers should be set to NULL first in the constructor
void ScriptingEngine::reset()
{
  SAFE_DELETE(jcpp);
  OK = false;
}

///////////////////////////////////////////////////////////////////////////////

/*
**
*/

bool ScriptingEngine::init(const string &whiteGamePath, 
			   const string &blackGamePath)
{
  dprintf(("entered ScriptingEngine::init(%s, %s)\n", 
	   whiteGamePath.c_str(), blackGamePath.c_str()));
  JavaVMOption options[3];	
  int n = 0;
  
  //char classPath[1024];
  string classPath;
  char *envcp = getenv("CLASSPATH");
  if(envcp != NULL){ // CLASSPATH variable is set
    classPath = "-Djava.class.path=" + string(envcp) + 
      string(PATH_SEPERATOR_STR) 
      + "." + string(PATH_SEPERATOR_STR) + whiteGamePath + 
      string(PATH_SEPERATOR_STR) + blackGamePath;
  }
  else {
    classPath = "-Djava.class.path=." + string(FILE_SEPERATOR_STR) + 
      string(PATH_SEPERATOR_STR) + whiteGamePath + 
      string(PATH_SEPERATOR_STR) + blackGamePath;
  }
  
  string libraryPath;
  libraryPath = "-Djava.library.path=" + whiteGamePath + 
    string(PATH_SEPERATOR_STR) + blackGamePath;  
  
  addOption(options, classPath, NULL); n++;
  addOption(options, libraryPath, NULL); n++;
  
  dprintf(("   numOptions: %d\n", n));
  OK = createJavaVM(options, n);
  dprintf(("leaving ScriptingEngine::initialize()\n"));
  return OK;
}

///////////////////////////////////////////////////////////////////////////////

ScriptController* ScriptingEngine::newScriptController(const string &className)
{	
  dprintf(("entered ScriptingEngine::newScriptController(%s)\n", 
	   className.c_str()));
  if(!isOK()) return NULL;	
  
  // shouldn't need this if this has already been initialized
  if(jcpp == NULL){
    dprintf(("  jvm == NULL\n"));
    return NULL;
  }
  
  dprintf(("leaving ScriptingEngine::newScriptController(%s)\n", 
	   className.c_str()));
  return new ScriptController(jcpp, className);  
}

///////////////////////////////////////////////////////////////////////////////


/*
** create and initialize the JavaVM_Mgr for this ScriptingEngine if
** it has not already been created
*/

bool ScriptingEngine::createJavaVM(JavaVMOption *options, int numOptions)
{ 
  dprintf(("entered ScriptingEngine::createJavaVM()\n"));	
  
  // should only be called during initialization
  if(isOK()) return false;
  
  jcpp = new JavaCPP();		
  if(!jcpp->init(options, numOptions)){
    dprintf(("Error: Could not initialize the java VM\n"));
    return false;
  }
  dprintf(("leaving ScriptingEngine::createJavaVM()\n"));
  return true;
}

///////////////////////////////////////////////////////////////////////////////


/*
** Adds a new VM option with the given given name and value.
** (borrowed from Sun's launcher/java.c code, converted to C++)
 */

int ScriptingEngine::addOption(JavaVMOption *options, const string &str, 
			       void *info)
{
  static int numOptions=0, maxOptions=9;
  /*
   * Expand options array if needed to accomodate at least one more
   * VM option. NOTE: ** don't think the expansion works...?
   */
  if (numOptions >= maxOptions) {
    if (options == NULL) {
      maxOptions = 4;	    						            
      options = new JavaVMOption[maxOptions*sizeof(JavaVMOption)];
    } 
    else {
      JavaVMOption *tmp;
      maxOptions += 2;	    
      tmp = new JavaVMOption[maxOptions * sizeof(JavaVMOption)];
      memcpy(tmp, options, numOptions * sizeof(JavaVMOption));	    
      delete options;
      options = tmp;
    }
  }
  options[numOptions].optionString = (char *)str.c_str();
  options[numOptions].extraInfo = info;
  return ++numOptions;
}

///////////////////////////////////////////////////////////////////////////////



