//
// JObject.cpp
//

#include <cstring>
#include "JObject.h"

#include "debug.h"
#include "JavaCPP.h"


//////////////////////////////////////////////////////////////////////////////


JObject::JObject(JNIEnv *env, const char *className, 
		 const char *constructorSig, va_list args)
{
  dprintf(("entered JObject(%s, %s)\n", className, constructorSig));
  clazz = NULL;
  object = NULL;
  OK = false;
  
  char *name = new char[strlen(className) + 1];
  char *s = name;
  const char *t = className;
  char c;	
  
  // convert '.' in class name to path slashes
  
  do{
    c = *t++;
    *s++ = (c == '.') ? '/' : c;		
  } while (c != '\0');
  
  // find the class the user wants to create
  dprintf(("  about to FindClass: %s .\n", name));
  clazz = env->FindClass(name);
  if(clazz == NULL){
    dprintf((" ** could not FindClass()\n"));
    if(env->ExceptionCheck()) env->ExceptionDescribe();
    return;  // should throw a C++ exception		
  }	
  
  
  // find the methodID of the contructor which matches
  // the <siganture argument
  jmethodID methodID = NULL;
  dprintf(("  about to GetMethodID for contructor: %s\n", constructorSig));
  methodID = env->GetMethodID(clazz, "<init>", constructorSig);
  if(methodID == NULL){
    if(env->ExceptionCheck()) env->ExceptionDescribe();
    return; // should throw a C++ exception		
  }
  
  dprintf(("  about to env->NewObject()\n"));
  jobject obj = env->NewObjectV(clazz, methodID, args);  
  
  if(env->ExceptionCheck()) env->ExceptionDescribe();
  if(obj == NULL) return;
  
  dprintf(("     * done *\n"));
  
  this->env = env;
  this->className = name;	
  this->object = obj;
  
  // making them a global ref tells the garbage collector 
  // not destroy which allows us to save 'em
  env->NewGlobalRef(object);	
  env->NewGlobalRef(clazz);
  
  OK = true;
  dprintf(("leaving JObject(%s, %s)\n", className, constructorSig));	
}

//////////////////////////////////////////////////////////////////////////////


JObject::JObject(JNIEnv *env, jobject obj)
{
  OK = false;
  if(env == NULL || obj == NULL) return;
  this->env = env;
  this->object = obj;
  this->clazz = env->GetObjectClass(object);
	
  env->NewGlobalRef(this->object);
  env->NewGlobalRef(this->clazz);
  OK = true;
}


//////////////////////////////////////////////////////////////////////////////


JObject::~JObject()
{
  if(clazz) env->DeleteGlobalRef(clazz);
  if(object) env->DeleteGlobalRef(object);	
  OK = false;
}

//////////////////////////////////////////////////////////////////////////////


jmethodID JObject::getMethodID(char *methodName, char *methodSig)
{
  return env->GetMethodID(clazz, methodName, methodSig);	
}

//////////////////////////////////////////////////////////////////////////////


void JObject::callVoidMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  env->CallVoidMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
}

//////////////////////////////////////////////////////////////////////////////


JObject *JObject::callObjectMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jobject obj = env->CallObjectMethodV(this->object, methodID, args);	
  
  // so we don't try to contruct a JObject with a null jobject
  if(env->ExceptionCheck()) env->ExceptionDescribe();
  if(obj == NULL){		
    return NULL;
  }
  va_end(args);		
  return new JObject(env, obj);
}

//////////////////////////////////////////////////////////////////////////////

jboolean JObject::callBooleanMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  bool result = (env->CallBooleanMethodV(this->object, methodID, args) == 1);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

jbyte JObject::callByteMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jbyte result = env->CallByteMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

jchar JObject::callCharMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jchar result = env->CallCharMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

jshort JObject::callShortMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jshort result = env->CallShortMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

jint JObject::callIntMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jint result = env->CallIntMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

jlong JObject::callLongMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jlong result = env->CallLongMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

jfloat JObject::callFloatMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jfloat result = env->CallFloatMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

jdouble JObject::callDoubleMethod(jmethodID methodID, ...)
{
  va_list args;
  va_start(args, methodID);
  jdouble result = env->CallDoubleMethodV(this->object, methodID, args);
  va_end(args);
  if(env->ExceptionCheck()) env->ExceptionDescribe();		
  return result;
}

//////////////////////////////////////////////////////////////////////////////

